"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ListrMultilineRenderer = require('listr-multiline-renderer');
const Listr = require('listr');
const operators_1 = require("rxjs/operators");
const hg_1 = require("../hg");
function multimerge(config, options) {
    const currentProject = config.projects.find(p => p.path === options.cwd);
    const projectDefaultBranch = (currentProject || { defaultBranch: null }).defaultBranch || 'default';
    const listrOpts = {
        concurrent: false,
        renderer: ListrMultilineRenderer,
        collapse: true
    };
    const pluralisedBranches = options.branchFragments.length === 1 ? 'branch' : 'branches';
    const tasks = new Listr([{
            title: `Multi-merging ${options.branchFragments.length} ${pluralisedBranches} in ${options.cwd}`,
            task: (ctx) => {
                ctx.allBranches = [];
                ctx.branches = [];
                ctx.numMerged = 0;
                return new Listr([
                    {
                        title: 'Preparing...',
                        task: () => {
                            return new Listr([
                                {
                                    title: `Ensuring no uncommitted changes`,
                                    task: () => {
                                        return hg_1.getStatus(options.cwd).pipe(operators_1.filter(Boolean), operators_1.tap(_ => {
                                            throw new Error(`Detected uncommitted changes! Please commit your work before running multimerge.`);
                                        }));
                                    }
                                },
                                {
                                    title: `Pulling changes`,
                                    task: () => hg_1.pullChanges(options.cwd)
                                },
                                {
                                    title: `Finding all branches`,
                                    task: (ctx) => {
                                        const omitInactive = true;
                                        return hg_1.getAllBranches(options.cwd, omitInactive)
                                            .pipe(operators_1.tap(branches => ctx.allBranches = branches));
                                    }
                                },
                                {
                                    title: `Matching fragments to branches`,
                                    task: () => {
                                        const fragmentMatcherTasks = options.branchFragments.map(fragment => {
                                            return {
                                                title: `Finding branch for ${fragment}`,
                                                task: (ctx) => {
                                                    const matches = ctx.allBranches.filter(branch => branch.toLowerCase().indexOf(fragment.toLowerCase()) !== -1);
                                                    if (!matches || !matches.length) {
                                                        throw new Error(`Unable to find a branch matching ${fragment}`);
                                                    }
                                                    if (matches.length > 1) {
                                                        const branchesMsg = matches.join('\n');
                                                        throw new Error(`Found multiple branches matching ${fragment}:\n${branchesMsg}`);
                                                    }
                                                    ctx.hasFoundBranchesToMerge = true;
                                                    ctx.branches.push(matches[0]);
                                                }
                                            };
                                        });
                                        return new Listr(fragmentMatcherTasks, Object.assign({}, listrOpts, { concurrent: true, exitOnError: false }));
                                    }
                                },
                            ], Object.assign({}, listrOpts, { exitOnError: true }));
                        }
                    },
                    {
                        title: 'Performing merges',
                        skip: (ctx) => !ctx.hasFoundBranchesToMerge,
                        task: (ctx) => {
                            const branchMergerTasks = ctx.branches.map(branch => {
                                return {
                                    title: branch,
                                    task: (ctx) => {
                                        return new Listr([
                                            {
                                                title: 'Update to branch',
                                                task: () => hg_1.update(options.cwd, branch, ['-C'])
                                            },
                                            {
                                                title: 'Do merge',
                                                task: () => {
                                                    let taskObs;
                                                    if (config.multimergeCmd) {
                                                        taskObs = hg_1.runArbitraryHgCommand(options.cwd, config.multimergeCmd);
                                                    }
                                                    else {
                                                        taskObs = hg_1.mergeBranchIn(options.cwd, projectDefaultBranch);
                                                    }
                                                    return taskObs.pipe(operators_1.catchError(err => {
                                                        ctx.skipCommit = true;
                                                        throw err;
                                                    }));
                                                }
                                            },
                                            {
                                                title: 'Commit',
                                                skip: (ctx) => ctx.skipCommit,
                                                task: (ctx) => {
                                                    ctx.numMerged++;
                                                    return hg_1.commit(options.cwd, 'Merged');
                                                }
                                            }
                                        ], Object.assign({}, listrOpts, { exitOnError: false }));
                                    }
                                };
                            });
                            return new Listr([
                                ...branchMergerTasks,
                                {
                                    title: `Updating back to default`,
                                    task: () => hg_1.update(options.cwd, projectDefaultBranch, ['-C'])
                                }
                            ], Object.assign({}, listrOpts, { exitOnError: false }));
                        }
                    },
                    {
                        title: 'Summary',
                        task: (ctx, task) => {
                            let summaryMsg;
                            if (!ctx.numMerged) {
                                summaryMsg = `nothing was merged...`;
                            }
                            else {
                                if (ctx.numMerged === ctx.branches.length) {
                                    summaryMsg = `all branches merged.`;
                                }
                                else {
                                    summaryMsg = `${ctx.numMerged} of ${ctx.branches.length} ${pluralisedBranches} merged.`;
                                }
                                summaryMsg += ` Don't forget to push!`;
                            }
                            task.title = `All done! ${summaryMsg}`;
                        }
                    }
                ], listrOpts);
            }
        }], listrOpts);
    return tasks.run();
}
exports.multimerge = multimerge;
//# sourceMappingURL=multimerge.js.map