"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const child_process = require("child_process");
const rxjs_1 = require("rxjs");
const operators_1 = require("rxjs/operators");
exports.MAX_CONCURRENT_HGS = 3;
function getBranch(projPath) {
    return rxjs_1.Observable.create((subscriber) => {
        const spawnOpts = { cwd: path.resolve(projPath) };
        subscriber.next('Running hg branch...');
        hgCommandToObservable(subscriber, ['branch'], spawnOpts);
    });
}
exports.getBranch = getBranch;
function getAllBranches(projPath, omitInactive = false) {
    return rxjs_1.Observable.create((subscriber) => {
        const spawnOpts = { cwd: path.resolve(projPath) };
        hgCommandToObservable(subscriber, ['branches'], spawnOpts);
    }).pipe(operators_1.mergeMap((branchesBlock) => {
        return branchesBlock.split('\n');
    }), operators_1.filter((branch) => {
        return !omitInactive
            || branch.indexOf('(inactive)') === -1;
    }), operators_1.map((branch) => {
        return branch.split(' ')[0];
    }), operators_1.reduce((acc, val) => {
        acc.push(val);
        return acc;
    }, []));
}
exports.getAllBranches = getAllBranches;
function getStatus(projPath) {
    return rxjs_1.Observable.create((subscriber) => {
        const spawnOpts = { cwd: path.resolve(projPath) };
        hgCommandToObservable(subscriber, ['status'], spawnOpts);
    });
}
exports.getStatus = getStatus;
function update(projPath, branchName, additionalArgs) {
    return rxjs_1.Observable.create((subscriber) => {
        const spawnOpts = { cwd: path.resolve(projPath) };
        subscriber.next('Updating to branch...');
        let hgArgs = ['up'];
        if (branchName) {
            hgArgs.push(branchName);
        }
        if (additionalArgs) {
            hgArgs = hgArgs.concat(additionalArgs);
        }
        hgCommandToObservable(subscriber, hgArgs, spawnOpts);
    });
}
exports.update = update;
function pullChanges(projPath) {
    return rxjs_1.Observable.create((subscriber) => {
        const spawnOpts = { cwd: path.resolve(projPath) };
        subscriber.next('Running hg pull...');
        hgCommandToObservable(subscriber, ['pull'], spawnOpts);
    });
}
exports.pullChanges = pullChanges;
function pushChanges(projPath) {
    return rxjs_1.Observable.create((subscriber) => {
        const spawnOpts = { cwd: path.resolve(projPath) };
        subscriber.next('Running hg push...');
        hgCommandToObservable(subscriber, ['push'], spawnOpts);
    });
}
exports.pushChanges = pushChanges;
function checkIfBranchNeedsOtherBranchMergingIn(projPath, branchName, defaultBranch) {
    return rxjs_1.Observable.create((subscriber) => {
        let hasFoundRevisions = false;
        rxjs_1.Observable.create((innerSubscriber) => {
            const spawnOpts = { cwd: path.resolve(projPath) };
            const revQuery = `not ancestors(heads(${branchName})) and head() and not closed() and ${defaultBranch}`;
            hgCommandToObservable(innerSubscriber, ['log', '-r', revQuery], spawnOpts);
        }).subscribe((data) => {
            hasFoundRevisions = !!(data || '').trim().length;
        }, (err) => {
            subscriber.error(err);
        }, () => {
            subscriber.next(hasFoundRevisions);
            subscriber.complete();
        });
    });
}
exports.checkIfBranchNeedsOtherBranchMergingIn = checkIfBranchNeedsOtherBranchMergingIn;
function mergeBranchIn(projPath, branchName) {
    return rxjs_1.Observable.create((subscriber) => {
        const spawnOpts = { cwd: path.resolve(projPath) };
        subscriber.next(`Merging in ${branchName}...`);
        hgCommandToObservable(subscriber, ['merge', branchName], spawnOpts);
    });
}
exports.mergeBranchIn = mergeBranchIn;
function commit(projPath, message) {
    return rxjs_1.Observable.create((subscriber) => {
        const spawnOpts = { cwd: path.resolve(projPath) };
        subscriber.next(`Committing...`);
        hgCommandToObservable(subscriber, ['com', '-m', message], spawnOpts);
    });
}
exports.commit = commit;
function runArbitraryHgCommand(projPath, ...command) {
    return rxjs_1.Observable.create((subscriber) => {
        const spawnOpts = { cwd: path.resolve(projPath) };
        hgCommandToObservable(subscriber, command, spawnOpts);
    });
}
exports.runArbitraryHgCommand = runArbitraryHgCommand;
function hgCommandToObservable(subscriber, hgArgs, spawnOpts) {
    let proc = child_process.spawn(`hg`, hgArgs, spawnOpts);
    proc.setMaxListeners(0);
    proc.stdout.on('data', buffer => {
        subscriber.next(buffer.toString());
    });
    proc.stderr.on('data', buffer => subscriber.error(new Error(buffer.toString())));
    proc.on('error', err => {
        subscriber.error(err);
        subscriber.complete();
    });
    proc.on('exit', _ => {
        subscriber.complete();
    });
    process.on('exit', () => proc.kill());
}
//# sourceMappingURL=hg.js.map